@@ -171,6 +171,9 @@ static gint               sort_by_date_deleted                    (const ThunarF
 static gint               sort_by_recency                         (const ThunarFile       *a,
                                                                    const ThunarFile       *b,
                                                                    gboolean                case_sensitive);
+static gint               sort_by_location                        (const ThunarFile       *a,
+                                                                   const ThunarFile       *b,
+                                                                   gboolean                case_sensitive);
 static gint               sort_by_group                           (const ThunarFile       *a,
                                                                    const ThunarFile       *b,
                                                                    gboolean                case_sensitive);
@@ -470,7 +473,7 @@ static void
 thunar_list_model_dispose (GObject *object)
 {
   /* unlink from the folder (if any) */
-  thunar_list_model_set_folder (THUNAR_LIST_MODEL (object), NULL);
+  thunar_list_model_set_folder (THUNAR_LIST_MODEL (object), NULL, NULL);
 
   (*G_OBJECT_CLASS (thunar_list_model_parent_class)->dispose) (object);
 }
@@ -566,7 +569,7 @@ thunar_list_model_set_property (GObject      *object,
       break;
 
     case PROP_FOLDER:
-      thunar_list_model_set_folder (store, g_value_get_object (value));
+      thunar_list_model_set_folder (store, g_value_get_object (value), NULL);
       break;
 
     case PROP_FOLDERS_FIRST:
@@ -623,6 +626,12 @@ thunar_list_model_get_column_type (GtkTreeModel *model,
     case THUNAR_COLUMN_DATE_DELETED:
       return G_TYPE_STRING;
 
+    case THUNAR_COLUMN_RECENCY:
+      return G_TYPE_STRING;
+
+    case THUNAR_COLUMN_LOCATION:
+      return G_TYPE_STRING;
+
     case THUNAR_COLUMN_GROUP:
       return G_TYPE_STRING;
 
@@ -720,6 +729,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
   ThunarFile  *file;
   GFile       *g_file;
   gchar       *str;
+  gchar       *uri;
 
   _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (model));
   _thunar_return_if_fail (iter->stamp == (THUNAR_LIST_MODEL (model))->stamp);
@@ -759,6 +769,14 @@ thunar_list_model_get_value (GtkTreeModel *model,
       g_value_take_string (value, str);
       break;
 
+    case THUNAR_COLUMN_LOCATION:
+      g_value_init (value, G_TYPE_STRING);
+      uri = thunar_file_dup_uri (file);
+      str = g_path_get_dirname (uri);
+      g_value_take_string (value, str);
+      g_free (uri);
+      break;
+
     case THUNAR_COLUMN_GROUP:
       g_value_init (value, G_TYPE_STRING);
       group = thunar_file_get_group (file);
@@ -1014,6 +1032,8 @@ thunar_list_model_get_sort_column_id (GtkTreeSortable *sortable,
     *sort_column_id = THUNAR_COLUMN_DATE_DELETED;
   else if (store->sort_func == sort_by_recency)
     *sort_column_id = THUNAR_COLUMN_RECENCY;
+  else if (store->sort_func == sort_by_location)
+    *sort_column_id = THUNAR_COLUMN_LOCATION;
   else if (store->sort_func == sort_by_type)
     *sort_column_id = THUNAR_COLUMN_TYPE;
   else if (store->sort_func == sort_by_owner)
@@ -1067,6 +1087,10 @@ thunar_list_model_set_sort_column_id (GtkTreeSortable *sortable,
       store->sort_func = sort_by_recency;
       break;
 
+    case THUNAR_COLUMN_LOCATION:
+      store->sort_func = sort_by_location;
+      break;
+
     case THUNAR_COLUMN_GROUP:
       store->sort_func = sort_by_group;
       break;
@@ -1319,7 +1343,7 @@ thunar_list_model_folder_destroy (ThunarFolder    *folder,
   _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (store));
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
 
-  thunar_list_model_set_folder (store, NULL);
+  thunar_list_model_set_folder (store, NULL, NULL);
 
   /* TODO: What to do when the folder is deleted? */
 }
@@ -1339,7 +1363,7 @@ thunar_list_model_folder_error (ThunarFolder    *folder,
   g_signal_emit (G_OBJECT (store), list_model_signals[ERROR], 0, error);
 
   /* reset the current folder */
-  thunar_list_model_set_folder (store, NULL);
+  thunar_list_model_set_folder (store, NULL, NULL);
 }
 
 
@@ -1539,6 +1563,35 @@ sort_by_recency      (const ThunarFile *a,
 
 
 
+static gint
+sort_by_location (const ThunarFile *a,
+                  const ThunarFile *b,
+                  gboolean          case_sensitive)
+{
+  gchar *uri_a;
+  gchar *uri_b;
+  gchar *location_a;
+  gchar *location_b;
+  gint   result;
+
+  uri_a = thunar_file_dup_uri (a);
+  uri_b = thunar_file_dup_uri (b);
+
+  location_a = g_path_get_dirname (uri_a);
+  location_b = g_path_get_dirname (uri_b);
+
+  result = strcasecmp (location_a, location_b);
+
+  g_free (uri_a);
+  g_free (uri_b);
+  g_free (location_a);
+  g_free (location_b);
+
+  return result;
+}
+
+
+
 static gint
 sort_by_group (const ThunarFile *a,
                const ThunarFile *b,
@@ -1965,18 +2018,68 @@ thunar_list_model_get_folder (ThunarListModel *store)
 
 
 
+static GList*
+search_directory (GList       *files,
+                  ThunarFile  *directory,
+                  gchar       *search_query_c)
+{
+  ThunarFolder *folder;
+  GList        *temp_files;
+  const gchar  *display_name;
+  gchar        *display_name_c; /* converted to ignore case */
+  gchar        *uri;
+
+  folder     = thunar_folder_get_for_file (directory);
+  temp_files = thunar_folder_get_files (folder);
+
+  for (; temp_files != NULL; temp_files = temp_files->next)
+    {
+      /* don't allow duplicates for files that exist in `recent:///` */
+      uri = thunar_file_dup_uri (temp_files->data);
+      if (gtk_recent_manager_has_item (gtk_recent_manager_get_default(), uri) == TRUE)
+        {
+          g_free (uri);
+          continue;
+        }
+
+      /* prepare entry display name */
+      display_name = thunar_file_get_display_name (temp_files->data);
+      display_name_c = g_utf8_casefold (display_name, strlen (display_name));
+
+      /* search substring */
+      if (g_strrstr (display_name_c, search_query_c) != NULL)
+        {
+          files = g_list_prepend (files, temp_files->data);
+          g_object_ref (temp_files->data);
+        }
+
+      /* free memory */
+      g_free (uri);
+      g_free (display_name_c);
+    }
+
+  g_object_unref (folder);
+
+  return files;
+}
+
+
+
 /**
  * thunar_list_model_set_folder:
- * @store  : a valid #ThunarListModel.
- * @folder : a #ThunarFolder or %NULL.
+ * @store                       : a valid #ThunarListModel.
+ * @folder                      : a #ThunarFolder or %NULL.
+ * @search_query                : a #string or %NULL.
  **/
 void
 thunar_list_model_set_folder (ThunarListModel *store,
-                              ThunarFolder    *folder)
+                              ThunarFolder    *folder,
+                              gchar           *search_query)
 {
   GtkTreePath   *path;
   gboolean       has_handler;
   GList         *files;
+  GList         *search_files; /* used to free results of a search */
   GSequenceIter *row;
   GSequenceIter *end;
   GSequenceIter *next;
@@ -1984,9 +2087,7 @@ thunar_list_model_set_folder (ThunarListModel *store,
   _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (store));
   _thunar_return_if_fail (folder == NULL || THUNAR_IS_FOLDER (folder));
 
-  /* check if we're not already using that folder */
-  if (G_UNLIKELY (store->folder == folder))
-    return;
+  search_files = NULL;
 
   /* unlink from the previously active folder (if any) */
   if (G_LIKELY (store->folder != NULL))
@@ -2042,13 +2143,59 @@ thunar_list_model_set_folder (ThunarListModel *store,
     {
       g_object_ref (G_OBJECT (folder));
 
-      /* get the already loaded files */
-      files = thunar_folder_get_files (folder);
+      /* get the already loaded files or search for files matching the search_query */
+      if (search_query == NULL)
+        {
+          files = thunar_folder_get_files (folder);
+        }
+      else
+        {
+          GList       *recent_infos;
+          GList       *lp;
+          gchar       *search_query_c; /* converted to ignore case */
+          const gchar *display_name;
+          gchar       *display_name_c; /* converted to ignore case */
+
+          recent_infos   = gtk_recent_manager_get_items (gtk_recent_manager_get_default ());
+          search_query_c = g_utf8_casefold (search_query, strlen (search_query));
+          files = NULL;
+
+          /* search the current folder */
+          files = search_directory (files, thunar_folder_get_corresponding_file (folder), search_query_c);
+
+          /* search GtkRecent */
+          for (lp = recent_infos; lp != NULL; lp = lp->next)
+            {
+              if (!gtk_recent_info_exists (lp->data))
+                continue;
+
+              /* prepare entry display name */
+              display_name = gtk_recent_info_get_display_name (lp->data);
+              display_name_c = g_utf8_casefold (display_name, strlen (display_name));
+
+              /* search substring */
+              if (g_strrstr (display_name_c, search_query_c) != NULL)
+                files = g_list_prepend (files, thunar_file_get_for_uri (gtk_recent_info_get_uri (lp->data), NULL));
+
+              /* free memory */
+              g_free (display_name_c);
+            }
+
+          search_files = files;
+          g_list_free_full (recent_infos, (void (*) (void*)) gtk_recent_info_unref);
+        }
 
       /* insert the files */
       if (files != NULL)
         thunar_list_model_files_added (folder, files, store);
 
+      /* free search files, thunar_list_model_files_added has added its own references */
+      if (search_files != NULL)
+        {
+          thunar_g_list_free_full (search_files);
+          search_files = NULL;
+        }
+
       /* connect signals to the new folder */
       g_signal_connect (G_OBJECT (store->folder), "destroy", G_CALLBACK (thunar_list_model_folder_destroy), store);
       g_signal_connect (G_OBJECT (store->folder), "error", G_CALLBACK (thunar_list_model_folder_error), store);
