@@ -45,6 +45,7 @@
 #include <thunar/thunar-path-entry.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-util.h>
+#include <thunar/thunar-window.h>
 
 
 
@@ -142,6 +143,8 @@ struct _ThunarPathEntry
   guint              in_change : 1;
   guint              has_completion : 1;
   guint              check_completion_idle_id;
+
+  gboolean           search_mode;
 };
 
 
@@ -272,6 +275,9 @@ thunar_path_entry_init (ThunarPathEntry *path_entry)
   /* connect the icon signals */
   g_signal_connect (G_OBJECT (path_entry), "icon-press", G_CALLBACK (thunar_path_entry_icon_press_event), NULL);
   g_signal_connect (G_OBJECT (path_entry), "icon-release", G_CALLBACK (thunar_path_entry_icon_release_event), NULL);
+
+  /* disabled initially */
+  path_entry->search_mode = FALSE;
 }
 
 
@@ -487,6 +493,13 @@ thunar_path_entry_key_press_event (GtkWidget   *widget,
       /* we handled the event */
       return TRUE;
     }
+  /* cancel search with `Escape` */
+  if (G_UNLIKELY (path_entry->search_mode == TRUE && event->keyval == GDK_KEY_Escape && (event->state & GDK_CONTROL_MASK) == 0))
+    {
+      GtkWidget *window = gtk_widget_get_toplevel (widget);
+      thunar_window_action_cancel_search (THUNAR_WINDOW (window));
+      return TRUE;
+    }
 
   return FALSE;
 }
@@ -560,33 +573,51 @@ thunar_path_entry_changed (GtkEditable *editable)
 
   /* parse the entered string (handling URIs properly) */
   text = gtk_entry_get_text (GTK_ENTRY (path_entry));
-  if (G_UNLIKELY (exo_str_looks_like_an_uri (text)))
-    {
-      /* try to parse the URI text */
-      escaped_text = g_uri_escape_string (text, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
-      file_path = g_file_new_for_uri (escaped_text);
-      g_free (escaped_text);
 
-      /* use the same file if the text assumes we're in a directory */
-      if (g_str_has_suffix (text, "/"))
-        folder_path = G_FILE (g_object_ref (G_OBJECT (file_path)));
-      else
-        folder_path = g_file_get_parent (file_path);
+  if (G_UNLIKELY (text != NULL && thunar_util_is_a_search_query (text)) == TRUE)
+    {
+      GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (editable));
+      path_entry->search_mode = TRUE;
+      update_icon = TRUE;
+      thunar_window_update_search (THUNAR_WINDOW (window));
     }
-  else if (thunar_path_entry_parse (path_entry, &folder_part, &file_part, NULL))
+  else
     {
-      /* determine the folder path */
-      folder_path = g_file_new_for_path (folder_part);
+      /* cancel search when part of 'Search: ' is deleted */
+      if (path_entry->search_mode == TRUE)
+        {
+          GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (editable));
+          thunar_window_action_cancel_search (THUNAR_WINDOW (window));
+        }
+      /* location/folder-path code */
+      if (G_UNLIKELY (exo_str_looks_like_an_uri (text)))
+        {
+          /* try to parse the URI text */
+          escaped_text = g_uri_escape_string (text, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
+          file_path = g_file_new_for_uri (escaped_text);
+          g_free (escaped_text);
 
-      /* determine the relative file path */
-      if (G_LIKELY (*file_part != '\0'))
-        file_path = g_file_resolve_relative_path (folder_path, file_part);
-      else
-        file_path = g_object_ref (folder_path);
+          /* use the same file if the text assumes we're in a directory */
+          if (g_str_has_suffix (text, "/"))
+            folder_path = G_FILE (g_object_ref (G_OBJECT (file_path)));
+          else
+            folder_path = g_file_get_parent (file_path);
+        }
+      else if (thunar_path_entry_parse (path_entry, &folder_part, &file_part, NULL))
+        {
+          /* determine the folder path */
+          folder_path = g_file_new_for_path (folder_part);
 
-      /* cleanup the part strings */
-      g_free (folder_part);
-      g_free (file_part);
+          /* determine the relative file path */
+          if (G_LIKELY (*file_part != '\0'))
+            file_path = g_file_resolve_relative_path (folder_path, file_part);
+          else
+            file_path = g_object_ref (folder_path);
+
+          /* cleanup the part strings */
+          g_free (folder_part);
+          g_free (file_part);
+        }
     }
 
   /* determine new current file/folder from the paths */
@@ -619,7 +650,7 @@ thunar_path_entry_changed (GtkEditable *editable)
       model = gtk_entry_completion_get_model (completion);
       g_object_ref (G_OBJECT (model));
       gtk_entry_completion_set_model (completion, NULL);
-      thunar_list_model_set_folder (THUNAR_LIST_MODEL (model), folder);
+      thunar_list_model_set_folder (THUNAR_LIST_MODEL (model), folder, NULL);
       gtk_entry_completion_set_model (completion, model);
       g_object_unref (G_OBJECT (model));
 
@@ -673,6 +704,14 @@ thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
   GtkIconTheme       *icon_theme;
   gint                icon_size;
 
+  if (path_entry->search_mode == TRUE)
+    {
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry),
+                                         GTK_ENTRY_ICON_PRIMARY,
+                                         "system-search");
+      return;
+    }
+
   if (path_entry->icon_factory == NULL)
     {
       icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (path_entry)));
@@ -1294,3 +1333,39 @@ thunar_path_entry_set_working_directory (ThunarPathEntry *path_entry,
   if (THUNAR_IS_FILE (working_directory))
     path_entry->working_directory = g_object_ref (thunar_file_get_file (working_directory));
 }
+
+
+
+/**
+ * thunar_path_entry_get_search_query:
+ * @path_entry        : a #ThunarPathEntry.
+ *
+ * Returns a copy of the search query in the text field of the @path_entry or NULL if the path_entry doesn't contain
+ * a search query.
+ *
+ * It's the responsibility of the caller to free the returned string using `g_free`.
+ **/
+gchar*
+thunar_path_entry_get_search_query (ThunarPathEntry *path_entry)
+{
+  unsigned long search_prefix_length;
+  const gchar *text = gtk_entry_get_text (GTK_ENTRY (path_entry));
+  search_prefix_length = strlen (SEARCH_PREFIX);
+
+  _thunar_return_val_if_fail (THUNAR_IS_PATH_ENTRY (path_entry), NULL);
+  _thunar_return_val_if_fail (strncmp (text, SEARCH_PREFIX, search_prefix_length) == 0, NULL);
+
+  return strlen(text) > search_prefix_length ? g_strdup (&text[search_prefix_length]) : NULL;
+}
+
+
+
+void
+thunar_path_entry_cancel_search (ThunarPathEntry *path_entry)
+{
+  _thunar_return_if_fail (THUNAR_IS_PATH_ENTRY (path_entry));
+
+  path_entry->search_mode = FALSE;
+
+  gtk_widget_grab_focus (gtk_widget_get_parent (GTK_WIDGET (path_entry)));
+}
