@@ -102,10 +102,13 @@ enum
   PROP_MISC_TEXT_BESIDE_ICONS,
   PROP_MISC_THUMBNAIL_MODE,
   PROP_MISC_THUMBNAIL_DRAW_FRAMES,
+  PROP_MISC_THUMBNAIL_MAX_FILE_SIZE,
   PROP_MISC_FILE_SIZE_BINARY,
   PROP_MISC_CONFIRM_CLOSE_MULTIPLE_TABS,
   PROP_MISC_PARALLEL_COPY_MODE,
   PROP_MISC_WINDOW_ICON,
+  PROP_MISC_TRANSFER_USE_PARTIAL,
+  PROP_MISC_TRANSFER_VERIFY_FILE,
   PROP_SHORTCUTS_ICON_EMBLEMS,
   PROP_SHORTCUTS_ICON_SIZE,
   PROP_TREE_ICON_EMBLEMS,
@@ -848,6 +851,19 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             FALSE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-thumbnail-max-file-size:
+   *
+   * Maximum file size (in bytes) allowed to be thumbnailed.
+   * 0 means no limit is in place.
+   **/
+  preferences_props[PROP_MISC_THUMBNAIL_MAX_FILE_SIZE] =
+      g_param_spec_uint64 ("misc-thumbnail-max-file-size",
+                           NULL,
+                           NULL,
+                           0, G_MAXUINT64, 0,
+                           EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:misc-file-size-binary:
    *
@@ -885,6 +901,32 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             TRUE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-transfer-use-partial:
+   *
+   * Whether to use intermediate file(*.partial~) to copy.
+   **/
+  preferences_props[PROP_MISC_TRANSFER_USE_PARTIAL] =
+    g_param_spec_enum ("misc-transfer-use-partial",
+                       "MiscTransferUsePartial",
+                       NULL,
+                       THUNAR_TYPE_USE_PARTIAL_MODE,
+                       THUNAR_USE_PARTIAL_MODE_DISABLED,
+                       EXO_PARAM_READWRITE);
+
+  /**
+   * ThunarPreferences:misc-transfer-verify-file:
+   *
+   * Whether to verify copied file with checksum.
+   **/
+  preferences_props[PROP_MISC_TRANSFER_VERIFY_FILE] =
+    g_param_spec_enum ("misc-transfer-verify-file",
+                       "MiscTransferVerifyFile",
+                       NULL,
+                       THUNAR_TYPE_VERIFY_FILE_MODE,
+                       THUNAR_VERIFY_FILE_MODE_DISABLED,
+                       EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:misc-confirm-close-multiple-tabs:
    *
