@@ -25,6 +25,7 @@
 
 #include <thunar/thunar-thumbnailer-proxy.h>
 #include <thunar/thunar-marshal.h>
+#include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-thumbnailer.h>
 
@@ -106,6 +107,7 @@ enum
 {
   PROP_0,
   PROP_THUMBNAIL_SIZE,
+  PROP_THUMBNAIL_MAX_FILE_SIZE,
 };
 
 
@@ -174,6 +176,9 @@ struct _ThunarThumbnailer
   /* size to use to store thumbnails */
   ThunarThumbnailSize thumbnail_size;
 
+  /* maximum file size (in bytes) allowed to be thumbnailed */
+  guint64     thumbnail_max_file_size;
+  
   /* IDs of idle functions */
   GSList     *idles;
 };
@@ -242,7 +247,7 @@ thunar_thumbnailer_class_init (ThunarThumbnailerClass *klass)
                   G_TYPE_NONE, 1, G_TYPE_UINT);
 
   /**
-   * ThunarIconFactory:thumbnail-size:
+   * ThunarThumbnailer:thumbnail-size:
    *
    * Size of the thumbnails to load
    **/
@@ -254,6 +259,19 @@ thunar_thumbnailer_class_init (ThunarThumbnailerClass *klass)
                                                       THUNAR_TYPE_THUMBNAIL_SIZE,
                                                       THUNAR_THUMBNAIL_SIZE_NORMAL,
                                                       EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarThumbnailer:thumbnail-max-file-size:
+   *
+   * Maximum file size (in bytes) allowed to be thumbnailed
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_THUMBNAIL_MAX_FILE_SIZE,
+                                   g_param_spec_uint64 ("thumbnail-max-file-size",
+                                                        "thumbnail-max-file-size",
+                                                        "thumbnail-max-file-size",
+                                                        0, G_MAXUINT64, 0,
+                                                        EXO_PARAM_READWRITE));
 }
 
 
@@ -272,6 +290,10 @@ thunar_thumbnailer_get_property (GObject    *object,
       g_value_set_enum (value, thumbnailer->thumbnail_size);
       break;
 
+    case PROP_THUMBNAIL_MAX_FILE_SIZE:
+      g_value_set_uint64 (value, thumbnailer->thumbnail_max_file_size);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -294,6 +316,10 @@ thunar_thumbnailer_set_property (GObject      *object,
       thumbnailer->thumbnail_size = g_value_get_enum (value);
       break;
 
+    case PROP_THUMBNAIL_MAX_FILE_SIZE:
+      thumbnailer->thumbnail_max_file_size = g_value_get_uint64 (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -425,6 +451,12 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
       if (thumb_state == THUNAR_FILE_THUMB_STATE_READY
           || thunar_thumbnailer_file_is_supported (thumbnailer, lp->data))
         {
+          guint max_size = thumbnailer->thumbnail_max_file_size;
+
+          /* skip large files */
+          if (max_size > 0 && thunar_file_get_size (lp->data) > max_size)
+            continue;
+
           supported_files = g_list_prepend (supported_files, lp->data);
           n_items++;
         }
@@ -515,6 +547,12 @@ thunar_thumbnailer_init (ThunarThumbnailer *thumbnailer)
 
   /* initialize the proxies */
   thunar_thumbnailer_init_thumbnailer_proxy (thumbnailer);
+
+  g_object_bind_property (G_OBJECT (thunar_preferences_get ()),
+                          "misc-thumbnail-max-file-size",
+                          G_OBJECT (thumbnailer),
+                          "thumbnail-max-file-size",
+                          G_BINDING_SYNC_CREATE);
 }
 
 
