@@ -48,6 +48,8 @@ enum
   PROP_0,
   PROP_FILE_SIZE_BINARY,
   PROP_PARALLEL_COPY_MODE,
+  PROP_TRANSFER_USE_PARTIAL,
+  PROP_TRANSFER_VERIFY_FILE,
 };
 
 
@@ -90,18 +92,20 @@ struct _ThunarTransferJob
   gchar                  *target_device_fs_id;
   gboolean                is_target_device_local;
 
-  gint64                  start_time;
-  gint64                  last_update_time;
-  guint64                 last_total_progress;
+  gint64                  start_time;              /* us(microseconds) */
+  gint64                  last_update_time;        /* us */
+  guint64                 last_total_progress;     /* byte */
 
-  guint64                 total_size;
-  guint64                 total_progress;
-  guint64                 file_progress;
-  guint64                 transfer_rate;
+  guint64                 total_size;              /* byte */
+  guint64                 total_progress;          /* byte */
+  guint64                 file_progress;           /* byte */
+  guint64                 transfer_rate;           /* byte/s */
 
   ThunarPreferences      *preferences;
   gboolean                file_size_binary;
   ThunarParallelCopyMode  parallel_copy_mode;
+  ThunarUsePartialMode    transfer_use_partial;
+  ThunarVerifyFileMode    transfer_verify_file;
 };
 
 struct _ThunarTransferNode
@@ -160,6 +164,34 @@ thunar_transfer_job_class_init (ThunarTransferJobClass *klass)
                                                       THUNAR_TYPE_PARALLEL_COPY_MODE,
                                                       THUNAR_PARALLEL_COPY_MODE_ONLY_LOCAL,
                                                       EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarPropertiesdialog:transfer_use_partial:
+   *
+   * Whether to use intermediate file to copy
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_TRANSFER_USE_PARTIAL,
+                                   g_param_spec_enum ("transfer-use-partial",
+                                                      "TransferUsePartial",
+                                                      NULL,
+                                                      THUNAR_TYPE_USE_PARTIAL_MODE,
+                                                      THUNAR_USE_PARTIAL_MODE_DISABLED,
+                                                      EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarPropertiesdialog:transfer_verify_file:
+   *
+   * Whether to verify copied file with checksum
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_TRANSFER_VERIFY_FILE,
+                                   g_param_spec_enum ("transfer-verify-file",
+                                                      "TransferVerifyFile",
+                                                      NULL,
+                                                      THUNAR_TYPE_VERIFY_FILE_MODE,
+                                                      THUNAR_VERIFY_FILE_MODE_DISABLED,
+                                                      EXO_PARAM_READWRITE));
 }
 
 
@@ -174,6 +206,12 @@ thunar_transfer_job_init (ThunarTransferJob *job)
   g_object_bind_property (job->preferences, "misc-parallel-copy-mode",
                           job,              "parallel-copy-mode",
                           G_BINDING_SYNC_CREATE);
+  g_object_bind_property (job->preferences, "misc-transfer-use-partial",
+                          job,              "transfer-use-partial",
+                          G_BINDING_SYNC_CREATE);
+  g_object_bind_property (job->preferences, "misc-transfer-verify-file",
+                          job,              "transfer-verify-file",
+                          G_BINDING_SYNC_CREATE);
 
   job->type = 0;
   job->source_node_list = NULL;
@@ -230,6 +268,12 @@ thunar_transfer_job_get_property (GObject     *object,
     case PROP_PARALLEL_COPY_MODE:
       g_value_set_enum (value, job->parallel_copy_mode);
       break;
+    case PROP_TRANSFER_USE_PARTIAL:
+      g_value_set_enum (value, job->transfer_use_partial);
+      break;
+    case PROP_TRANSFER_VERIFY_FILE:
+      g_value_set_enum (value, job->transfer_verify_file);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -254,6 +298,12 @@ thunar_transfer_job_set_property (GObject      *object,
     case PROP_PARALLEL_COPY_MODE:
       job->parallel_copy_mode = g_value_get_enum (value);
       break;
+    case PROP_TRANSFER_USE_PARTIAL:
+      job->transfer_use_partial = g_value_get_enum (value);
+      break;
+    case PROP_TRANSFER_VERIFY_FILE:
+      job->transfer_verify_file = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -305,7 +355,11 @@ thunar_transfer_job_progress (goffset  current_num_bytes,
       expired_time = current_time - job->last_update_time;
 
       /* notify callers not more then every 500ms */
-      if (expired_time > (500 * 1000))
+      /* force update after transfer when it took more than (approx.) 500ms */
+      /* the actual code checks if (file size [byte]) > (transfer rate [byte/s]) * (0.5 [s]) */
+      /* which means that the file is bigger than what is transferred in 500ms on average */
+      if (expired_time > (500 * 1000)
+          || (current_num_bytes == total_num_bytes && total_num_bytes > (goffset) (job->transfer_rate / 2)))
         {
           /* calculate the transfer rate in the last expired time */
           transfer_rate = (job->total_progress - job->last_total_progress) / ((gfloat) expired_time / G_USEC_PER_SEC);
@@ -415,6 +469,8 @@ ttj_copy_file (ThunarTransferJob *job,
   GFileType  source_type;
   GFileType  target_type;
   gboolean   target_exists;
+  gboolean   use_partial;
+  gboolean   verify_file;
   GError    *err = NULL;
 
   _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), FALSE);
@@ -454,10 +510,53 @@ ttj_copy_file (ThunarTransferJob *job,
         }
     }
 
+  switch (job->transfer_use_partial)
+    {
+    case THUNAR_USE_PARTIAL_MODE_REMOTE_ONLY:
+      use_partial = !g_file_is_native (source_file) || !g_file_is_native (target_file);
+      break;
+    case THUNAR_USE_PARTIAL_MODE_ALWAYS:
+      use_partial = TRUE;
+      break;
+    default:
+      use_partial = FALSE;
+    }
+
   /* try to copy the file */
-  g_file_copy (source_file, target_file, copy_flags,
-               exo_job_get_cancellable (EXO_JOB (job)),
-               thunar_transfer_job_progress, job, &err);
+  thunar_g_file_copy (source_file, target_file, copy_flags, use_partial,
+                      exo_job_get_cancellable (EXO_JOB (job)),
+                      thunar_transfer_job_progress, job, &err);
+
+  switch (job->transfer_verify_file)
+    {
+    case THUNAR_VERIFY_FILE_MODE_REMOTE_ONLY:
+      verify_file = !g_file_is_native (source_file) || !g_file_is_native (target_file);
+      break;
+    case THUNAR_VERIFY_FILE_MODE_ALWAYS:
+      verify_file = TRUE;
+      break;
+    default:
+      verify_file = FALSE;
+    }
+
+  /* Only verify when the file is a regular file */
+  verify_file = verify_file && (g_file_query_file_type (source_file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, exo_job_get_cancellable (EXO_JOB (job))) == G_FILE_TYPE_REGULAR);
+
+  if (verify_file && err == NULL)
+    {
+      gboolean is_equal;
+      exo_job_info_message (EXO_JOB (job), _("Comparing checksums..."));
+      is_equal = thunar_g_file_compare_checksum (source_file, target_file,
+                                                 exo_job_get_cancellable (EXO_JOB (job)), &err);
+
+      /* if the copied file is corrupted and yet no error*/
+      if (!is_equal && err == NULL)
+        {
+          err = g_error_new (G_FILE_ERROR,
+                             G_FILE_ERROR_AGAIN,
+                             "Copied file does not match with the original");
+        }
+    }
 
   /**
    * MR !127 notes:
@@ -586,7 +685,6 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
   GFile            *dest_file = target_file;
   GFileCopyFlags    copy_flags = G_FILE_COPY_NOFOLLOW_SYMLINKS;
   GError           *err = NULL;
-  gint              n;
   gint              n_rename = 0;
 
   _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), NULL);
@@ -601,43 +699,22 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
   /* various attempts to copy the file */
   while (err == NULL)
     {
+      GFile *target;
+
       thunar_transfer_job_check_pause (job);
+
       if (G_LIKELY (!g_file_equal (source_file, dest_file)))
-        {
-          /* try to copy the file from source_file to the dest_file */
-          if (ttj_copy_file (job, source_file, dest_file, copy_flags, TRUE, &err))
-            {
-              /* return the real target file */
-              return g_object_ref (dest_file);
-            }
-        }
+        target = g_object_ref (dest_file);
       else
+        target = thunar_io_jobs_util_next_duplicate_file (THUNAR_JOB (job), source_file, THUNAR_NEXT_FILE_NAME_MODE_COPY, &err);
+
+      if (err == NULL)
         {
-          for (n = 1; err == NULL; ++n)
-            {
-              GFile *duplicate_file = thunar_io_jobs_util_next_duplicate_file (THUNAR_JOB (job),
-                                                                               source_file,
-                                                                               TRUE, n,
-                                                                               &err);
-
-              if (err == NULL)
-                {
-                  /* try to copy the file from source file to the duplicate file */
-                  if (ttj_copy_file (job, source_file, duplicate_file, copy_flags, FALSE, &err))
-                    {
-                      /* return the real target file */
-                      return duplicate_file;
-                    }
-
-                  g_object_unref (duplicate_file);
-                }
-
-              if (err != NULL && err->domain == G_IO_ERROR && err->code == G_IO_ERROR_EXISTS)
-                {
-                  /* this duplicate already exists => clear the error to try the next alternative */
-                  g_clear_error (&err);
-                }
-            }
+          /* try to copy the file from source file to the duplicate file */
+          if (ttj_copy_file (job, source_file, target, copy_flags, FALSE, &err))
+            return target;
+          else /* go to error case */
+            g_object_unref (target);
         }
 
       /* check if we can recover from this error */
@@ -706,14 +783,18 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
                                GList             **target_file_list_return,
                                GError            **error)
 {
+  static const GRegex  *windows_reserved_name = NULL;
   ThunarThumbnailCache *thumbnail_cache;
   ThunarApplication    *application;
   ThunarJobResponse     response;
   GFileInfo            *info;
+  GFileInfo            *fs_info;
   GError               *err = NULL;
   GFile                *real_target_file = NULL;
   gchar                *base_name;
+  const gchar          *fs_type;
   gboolean              should_use_copy_name;
+  gboolean              use_fat_name_scheme;
 
   _thunar_return_if_fail (THUNAR_IS_TRANSFER_JOB (job));
   _thunar_return_if_fail (node != NULL && G_IS_FILE (node->source_file));
@@ -726,6 +807,14 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
    * wrt restoring files from the trash. Other transfer_nodes will be called with target_parent_file.
    */
 
+  /* if regex pattern is not initialized, do it */
+  if (G_UNLIKELY (windows_reserved_name == NULL))
+    {
+      /* COM#, LPT#, CON, PRN, AUX, and NUL are not allowed */
+      /* FAT is case-insensitive by default */
+      windows_reserved_name = g_regex_new ("^((COM\\d)|(LPT\\d)|(CON)|(PRN)|(AUX)|(NUL))(\\..*)?$", G_REGEX_CASELESS, 0, NULL);
+    }
+
   /* take a reference on the thumbnail cache */
   application = thunar_application_get ();
   thumbnail_cache = thunar_application_get_thumbnail_cache (application);
@@ -733,6 +822,32 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
 
   should_use_copy_name = G_UNLIKELY (!g_file_is_native (node->source_file));
 
+  if (target_parent_file == NULL)
+    target_parent_file = g_file_get_parent (target_file);
+  else
+    g_object_ref (target_parent_file);
+  g_assert (target_parent_file != NULL);
+
+  fs_info = g_file_query_filesystem_info (target_parent_file,
+                                          G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
+                                          NULL, NULL);
+  if (fs_info != NULL)
+    {
+      fs_type = g_file_info_get_attribute_string (fs_info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+      use_fat_name_scheme =
+        !strcmp (fs_type, "fat")   ||
+        !strcmp (fs_type, "vfat")  ||
+        !strcmp (fs_type, "fuse")  ||
+        !strcmp (fs_type, "ntfs")  ||
+        !strcmp (fs_type, "msdos") ||
+        !strcmp (fs_type, "msdosfs");
+    }
+  else
+    {
+      /* default to native */
+      use_fat_name_scheme = FALSE;
+    }
+
   for (; err == NULL && node != NULL; node = node->next)
     {
       /* query file info */
@@ -749,17 +864,12 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
       /* guess the target file for this node (unless already provided) */
       if (should_use_copy_name)
         {
-          if (target_parent_file == NULL)
-            target_parent_file = g_file_get_parent (target_file);
-          else
-            g_object_ref (target_parent_file);
           base_name = g_strdup (g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_COPY_NAME));
           /* copy name is NULLable, so use display name for fallback */
           if (base_name == NULL)
             base_name = g_strdup (g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME));
           target_file = g_file_get_child (target_parent_file, base_name);
           g_free (base_name);
-          g_object_unref (target_parent_file);
         }
       else if (G_LIKELY (target_file == NULL))
         {
@@ -770,6 +880,43 @@ thunar_transfer_job_copy_node (ThunarTransferJob  *job,
       else
         target_file = g_object_ref (target_file);
 
+      if (use_fat_name_scheme)
+        {
+          base_name = g_file_get_basename (target_file);
+          g_clear_object (&target_file);
+
+          /* replace invalid chars */
+          g_strdelimit (g_strchomp (base_name),
+                        "/:*?\"<>\\|",
+                        '_');
+
+          /* character 0~31 is invalid */
+          for (int i = 0; base_name[i] != '\0'; i++)
+            {
+              if (base_name[i] < 32)
+                base_name[i] = '_';
+            }
+
+          /* avoid reserved names */
+          if (g_regex_match (windows_reserved_name, base_name, 0, NULL))
+            {
+              gchar *tmp = base_name;
+              base_name = g_strconcat ("__", tmp, NULL);
+              g_free (tmp);
+            }
+
+          /* avoid filename that ends with '.' */
+          if (g_str_has_suffix (base_name, "."))
+            {
+              gchar *tmp = base_name;
+              base_name = g_strconcat (tmp, "___", NULL);
+              g_free (tmp);
+            }
+
+          target_file = g_file_get_child (target_parent_file, base_name);
+          g_free (base_name);
+        }
+
       /* update progress information */
       exo_job_info_message (EXO_JOB (job), "%s", g_file_info_get_display_name (info));
 
@@ -870,13 +1017,18 @@ retry_remove:
         }
 
       /* release the guessed target file */
-      g_object_unref (target_file);
-      target_file = NULL;
+      g_clear_object (&target_file);
 
       /* release file info */
       g_object_unref (info);
     }
 
+  /* release parent file */
+  g_object_unref (target_parent_file);
+
+  /* release filesystem info */
+  g_clear_object (&fs_info);
+
   /* release the thumbnail cache */
   g_object_unref (thumbnail_cache);
 
